/****************************************************************************
*                                     *                                     *
*  Jeff Molofee's IPicture Basecode   *    Huge Thanks To: Dave Richards    *
*       http://nehe.gamedev.net       *                    Bobby Ward &     *
*                2002                 *                    The MSDN         *
*                                     *                                     *
*****************************************************************************
*   Massive modifications by ZJWOOD 5/25/04
****************************************************************************/

#include <Cg/cg.h>														// NEW: Cg Header
#include <Cg/cgGL.h>													// NEW: Cg OpenGL Specific Header

#include <glut.h>
#include <GL/gl.h>

#include <stdlib.h>
#include <stdio.h>

// User Defined Variables
bool		cg_enable = true, sp;										// Toggle Cg Program On / Off, Space Pressed?

CGcontext	cgContext;													// A Context To Hold Our Cg Program(s)
CGprogram	cgProgram;													// Our Cg Vertex Program
CGprofile	cgVertexProfile;											// The Profile To Use For Our Vertex Shader
// The Parameters Needed For Our Shader
CGparameter	position, color, modelViewMatrix, modelViewIT, normal, LightVec;	


bool Initialize ()					// Any GL Init Code & User Initialiazation Goes Here
{

	// Start Of User Initialization
	glClearColor (0.0f, 0.0f, 0.0f, 0.5f);								// Black Background
	glClearDepth (1.0f);												// Depth Buffer Setup
	glDepthFunc (GL_LEQUAL);											// The Type Of Depth Testing
	glEnable (GL_DEPTH_TEST);											// Enable Depth Testing
	glShadeModel (GL_SMOOTH);											// Select Smooth Shading
	glHint (GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);					// Set Perspective Calculations To Most Accurate
	glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);							// Draw Our cube filled in

	// Setup Cg
	cgContext = cgCreateContext();										// Create A New Context For Our Cg Program(s)

	// Validate Our Context Generation Was Successful
	if (cgContext == 0)
	{
		fprintf(stderr, "Failed To Create Cg Context\n");
		exit(-1);													// We Cannot Continue
	}

	cgVertexProfile = cgGLGetLatestProfile(CG_GL_VERTEX);				// Get The Latest GL Vertex Profile

	// Validate Our Profile Determination Was Successful
	if (cgVertexProfile == CG_PROFILE_UNKNOWN)
	{
		fprintf(stderr, "Invalid profile type\n");
		exit(-1);													// We Cannot Continue
	}

	cgGLSetOptimalOptions(cgVertexProfile);								// Set The Current Profile

	// Load And Compile The Vertex Shader From File
	cgProgram = cgCreateProgramFromFile(cgContext, CG_SOURCE, "Cg/diffuse_shader.cg", cgVertexProfile, "main", 0);

	// Validate Success
	if (cgProgram == 0)
	{
		// We Need To Determine What Went Wrong
		CGerror Error = cgGetError();

		// Show A Message Box Explaining What Went Wrong
		fprintf(stderr,"%s \n",cgGetErrorString(Error));
		exit(-1);													// We Cannot Continue
	}

	// Load The Program
	cgGLLoadProgram(cgProgram);

	// Get Handles To Each Of Our Parameters So That
	// We Can Change Them At Will Within Our Code
	position		= cgGetNamedParameter(cgProgram, "IN.position");
	color			= cgGetNamedParameter(cgProgram, "IN.color");
	LightVec		= cgGetNamedParameter(cgProgram, "LightVec");
	modelViewMatrix	= cgGetNamedParameter(cgProgram, "ModelViewProj");
	modelViewIT = cgGetNamedParameter(cgProgram, "ModelViewIT");
	normal = cgGetNamedParameter(cgProgram, "normal");

	return true;														// Return true (Initialization Successful)
}

void Deinitialize (void)												// Any User DeInitialization Goes Here
{
	// Destroy Our Cg Context And All Programs Contained Within It
	cgDestroyContext(cgContext);
}

void Update (int key,int x,int y)										// Perform Motion Updates Here
{
	if ( key==27 )									// Is ESC Being Pressed?
	{
		Deinitialize();
		exit(0);							// Terminate The Program
	};
	
//	if (g_keys->keyDown [VK_F1])										// Is F1 Being Pressed?
//		ToggleFullscreen (g_window);									// Toggle Fullscreen Mode


	if ( key==' ' )
	{
		sp=!sp;
		cg_enable=!cg_enable;
	};


	glutPostRedisplay();
}

void DrawCube() {
	//left face: for lighting you must specify a normal for each face!
  glNormal3f(-1.0, 0.0, 0.0);
  glBegin(GL_POLYGON);
  glVertex3f(-0.5, 0.5, 0.5); //v5
  glVertex3f(-0.5, 0.5, -0.5); //v4
  glVertex3f(-0.5, -0.5, -0.5); //v1
  glVertex3f(-0.5, -0.5, 0.5); //v8
  glEnd();
  
  //right face:  for lighting you must specify a normal for each face!
  glNormal3f(1.0, 0.0, 0.0);
  glBegin(GL_POLYGON);
  glVertex3f(0.5, -0.5, -0.5); //v2
  glVertex3f(0.5, 0.5, -0.5); //v3
  glVertex3f(0.5, 0.5, 0.5); //v6
  glVertex3f(0.5, -0.5, 0.5); 
  glEnd();
  
  //bottom face:  for lighting you must specify a normal for each face!
  glNormal3f(0, -1.0, 0.0);
  glBegin(GL_POLYGON);
  glVertex3f(-0.5, -0.5, 0.5); //v8
  glVertex3f(0.5, -0.5, 0.5); //v7
  glVertex3f(0.5, -0.5, -0.5); //v2
  glVertex3f(-0.5, -0.5, -0.5); //v1
  glEnd();
  
  //top face:  for lighting you must specify a normal for each face!
  glNormal3f(0, 1.0, 0.0);
  glBegin(GL_POLYGON);
  glVertex3f(0.5, 0.5, -0.5); //v3
  glVertex3f(-0.5, 0.5, -0.5); //v4
  glVertex3f(-0.5, 0.5, 0.5); //v5
  glVertex3f(0.5, 0.5, 0.5); //v6
  glEnd();
  
  //front face:  for lighting you must specify a normal for each face!
  glNormal3f(0, 0, 1.0);
  glBegin(GL_POLYGON);
  glVertex3f(-0.5, 0.5, 0.5); //v5
  glVertex3f(0.5, 0.5, 0.5); //v6
  glVertex3f(0.5, -0.5, 0.5); //v7
  glVertex3f(-0.5, -0.5, 0.5); //v8
  glEnd();
  
  //back face:  for lighting you must specify a normal for each face!
  glNormal3f(0, 0, -1.0);
  glBegin(GL_POLYGON);
  glVertex3f(-0.5, -0.5, -0.5); //v1
  glVertex3f(0.5, -0.5, -0.5); //v2
  glVertex3f(0.5, 0.5, -0.5); //v3
  glVertex3f(-0.5, 0.5, -0.5); //v4
  glEnd();

}


void Draw (void)
{
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);				// Clear Screen And Depth Buffer
	glLoadIdentity ();													// Reset The Modelview Matrix

	// Position The Camera To Look At Our Mesh From A Distance
	gluLookAt(0.0f, 0.0f, 4.0f, 0.0f, 0.0f, 0.0f, 0, 1, 0);

	glRotatef(45, 0, 1, 0);
	glRotatef(45, 1, 0, 0);

	// Set The Modelview Matrix Of Our Shader To Our OpenGL Modelview Matrix
	cgGLSetStateMatrixParameter(modelViewMatrix, CG_GL_MODELVIEW_PROJECTION_MATRIX, CG_GL_MATRIX_IDENTITY);
	//cgGLSetStateMatrixParameter(modelViewMatrix, CG_GL_MODELVIEW_PROJECTION_MATRIX, CG_GL_MATRIX_IDENTITY);
	cgGLSetStateMatrixParameter(modelViewIT, CG_GL_MODELVIEW_MATRIX, CG_GL_MATRIX_INVERSE_TRANSPOSE);

	if (cg_enable)
	{
		cgGLEnableProfile(cgVertexProfile);								// Enable Our Vertex Shader Profile

		// Bind Our Vertex Program To The Current State
		cgGLBindProgram(cgProgram);

		// Set The Drawing Color To Light Green (Can Be Changed By Shader, Etc...)
		cgGLSetParameter4f(color, 1.0f, 0.1f, 0.1f, 1.0f);
		cgGLSetParameter3f(LightVec, 0.0, 0.0, 4.0);
	}

	DrawCube();

	if (cg_enable)
		cgGLDisableProfile(cgVertexProfile);							// Disable Our Vertex Profile
													
	glutSwapBuffers();
}

void ReshapeGL (int width, int height)									// Reshape The Window When It's Moved Or Resized
{
	glViewport (0, 0, (GLsizei)(width), (GLsizei)(height));				// Reset The Current Viewport
	glMatrixMode (GL_PROJECTION);										// Select The Projection Matrix
	glLoadIdentity ();													// Reset The Projection Matrix
	gluPerspective (60.0f, (GLfloat)(width)/(GLfloat)(height),			// Calculate The Aspect Ratio Of The Window
					0.1f, 5.0f);		
	glMatrixMode (GL_MODELVIEW);										// Select The Modelview Matrix
	glLoadIdentity ();													// Reset The Modelview Matrix

	glutPostRedisplay();
}


// GLUT 

void Key(unsigned char key,int x,int y)
{
	Update(key,x,y);
	return ;
};

void OnIdle()
{
	glutPostRedisplay();
};

int main( int argc, char *argv[] )
{
   glutInit( &argc, argv );
   glutInitWindowPosition( 20, 20 );
   glutInitWindowSize( 400, 400 );
   glutInitDisplayMode( GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH );
   glutCreateWindow("diffuse shader with no lights??");
   glutReshapeFunc( ReshapeGL );
   glutKeyboardFunc( Key );
   glutSpecialFunc( Update );
   glutDisplayFunc( Draw );
   Initialize();
   glutMainLoop();
   return 0;
}
